@@ -24,8 +24,8 @@ from contextlib import contextmanager
 
 
 from . import states
-from . import local
 from . import validator
+from .local import set_node_info, CurrentNodeInfo
 from .exceptions import InvalidOperationError, NotFoundError, StateVersionNotMatchError
 from .handler import HandlerFactory
 from .metrics import (
@@ -65,9 +65,7 @@ class Engine:
         self.runtime = runtime
 
     # api
-    def run_pipeline(
-        self, pipeline: dict, root_pipeline_data: Optional[dict] = None, **options
-    ):
+    def run_pipeline(self, pipeline: dict, root_pipeline_data: Optional[dict] = None, **options):
         """
         运行流程
 
@@ -83,9 +81,7 @@ class Engine:
 
         self.runtime.pre_prepare_run_pipeline(pipeline, root_pipeline_data, **options)
 
-        process_id = self.runtime.prepare_run_pipeline(
-            pipeline, root_pipeline_data, **options
-        )
+        process_id = self.runtime.prepare_run_pipeline(pipeline, root_pipeline_data, **options)
         # execute from start event
         self.runtime.execute(process_id, pipeline["start_event"]["id"])
 
@@ -133,9 +129,7 @@ class Engine:
         state = self.runtime.get_state(pipeline_id)
 
         if state.name != states.SUSPENDED:
-            raise InvalidOperationError(
-                "pipeline({}) state is: {}".format(pipeline_id, state.name)
-            )
+            raise InvalidOperationError("pipeline({}) state is: {}".format(pipeline_id, state.name))
 
         info_list = self.runtime.get_suspended_process_info(pipeline_id)
 
@@ -160,9 +154,7 @@ class Engine:
         node = self.runtime.get_node(node_id)
 
         if node.type == NodeType.SubProcess:
-            raise InvalidOperationError(
-                "can not use pause_node_appoint api for {}".format(node.type)
-            )
+            raise InvalidOperationError("can not use pause_node_appoint api for {}".format(node.type))
 
         self.runtime.pre_pause_node(node_id)
 
@@ -180,9 +172,7 @@ class Engine:
         node = self.runtime.get_node(node_id)
 
         if node.type == NodeType.SubProcess:
-            raise InvalidOperationError(
-                "can not use pause_node_appoint api for {}".format(node.type)
-            )
+            raise InvalidOperationError("can not use pause_node_appoint api for {}".format(node.type))
 
         self.runtime.pre_resume_node(node_id)
 
@@ -209,9 +199,7 @@ class Engine:
         node = self.runtime.get_node(node_id)
 
         if not node.can_retry:
-            raise InvalidOperationError(
-                "can not retry node({}) with can_retry({})".format(node_id, node.can)
-            )
+            raise InvalidOperationError("can not retry node({}) with can_retry({})".format(node_id, node.can))
 
         state = self.runtime.get_state(node_id)
 
@@ -255,9 +243,7 @@ class Engine:
             raise InvalidOperationError("can not skip this node")
 
         if node.type not in self.PURE_SKIP_ENABLE_NODE_TYPE:
-            raise InvalidOperationError(
-                "can not use skip_node api for {}".format(node.type)
-            )
+            raise InvalidOperationError("can not use skip_node api for {}".format(node.type))
 
         state = self.runtime.get_state(node_id)
 
@@ -297,11 +283,7 @@ class Engine:
         node = self.runtime.get_node(node_id)
 
         if node.type != NodeType.ExclusiveGateway:
-            raise InvalidOperationError(
-                "{} is not exclusive gateway, actual: {}".format(
-                    node_id, node.type.value
-                )
-            )
+            raise InvalidOperationError("{} is not exclusive gateway, actual: {}".format(node_id, node.type.value))
 
         next_node_id = node.targets[flow_id]
 
@@ -339,23 +321,17 @@ class Engine:
         node = self.runtime.get_node(node_id)
 
         if node.type != NodeType.ServiceActivity:
-            raise InvalidOperationError(
-                "{} is not activity, actual: {}".format(node_id, node.type.value)
-            )
+            raise InvalidOperationError("{} is not activity, actual: {}".format(node_id, node.type.value))
 
         state = self.runtime.get_state(node_id)
 
         if state.name != states.RUNNING:
-            raise InvalidOperationError(
-                "{} state is not RUNNING, actual: {}".format(node_id, state.name)
-            )
+            raise InvalidOperationError("{} state is not RUNNING, actual: {}".format(node_id, state.name))
 
         process_id = self.runtime.get_process_id_with_current_node_id(node_id)
 
         if not process_id:
-            raise InvalidOperationError(
-                "can not find process with current node id: {}".format(node_id)
-            )
+            raise InvalidOperationError("can not find process with current node id: {}".format(node_id))
 
         self.runtime.pre_forced_fail_activity(node_id, ex_data)
 
@@ -364,7 +340,8 @@ class Engine:
         outputs["ex_data"] = ex_data
         outputs["_forced_failed"] = True
 
-        self.runtime.set_state(
+        old_ver = state.version
+        new_ver = self.runtime.set_state(
             node_id=node_id,
             to_state=states.FAILED,
             refresh_version=True,
@@ -375,7 +352,7 @@ class Engine:
 
         self.runtime.kill(process_id)
 
-        self.runtime.post_forced_fail_activity(node_id, ex_data)
+        self.runtime.post_forced_fail_activity(node_id, ex_data, old_ver, new_ver)
 
     def callback(self, node_id: str, version: str, data: dict):
         """
@@ -396,9 +373,7 @@ class Engine:
         process_id = self.runtime.get_sleep_process_with_current_node_id(node_id)
 
         if not process_id:
-            raise InvalidOperationError(
-                "can not find process with current node id: {}".format(node_id)
-            )
+            raise InvalidOperationError("can not find process with current node id: {}".format(node_id))
 
         state = self.runtime.get_state(node_id)
 
@@ -448,20 +423,15 @@ class Engine:
             # 遇到推进终点后需要尝试唤醒父进程
             if current_node_id == process_info.destination_id:
                 self.runtime.die(process_id)
-                wake_up_seccess = self.runtime.child_process_finish(
-                    process_info.parent_id, process_id
-                )
+                wake_up_seccess = self.runtime.child_process_finish(process_info.parent_id, process_id)
 
                 if wake_up_seccess:
-                    self.runtime.execute(
-                        process_info.parent_id, process_info.destination_id
-                    )
+                    self.runtime.execute(process_info.parent_id, process_info.destination_id)
 
                 return
 
             try:
 
-                local.set_node_id(current_node_id)
                 self.runtime.set_current_node(process_id, current_node_id)
 
                 # 冻结检测
@@ -469,9 +439,7 @@ class Engine:
                     self.runtime.freeze(process_id)
                     return
 
-                node_state_map = self.runtime.batch_get_state_name(
-                    process_info.pipeline_stack
-                )
+                node_state_map = self.runtime.batch_get_state_name(process_info.pipeline_stack)
 
                 # 检测根流程是否被撤销
                 if node_state_map[process_info.root_pipeline_id] == states.REVOKED:
@@ -501,25 +469,17 @@ class Engine:
                 reset_mark_bit = False
 
                 if node_state:
-                    rerun_limit = self.runtime.node_rerun_limit(
-                        process_info.root_pipeline_id, current_node_id
-                    )
+                    rerun_limit = self.runtime.node_rerun_limit(process_info.root_pipeline_id, current_node_id)
                     # 重入次数超过限制
                     if (
                         node_state.name == states.FINISHED
                         and node.type != NodeType.SubProcess
                         and node_state.loop > rerun_limit
                     ):
-                        exec_outputs = self.runtime.get_execution_data_outputs(
-                            current_node_id
-                        )
-                        exec_outputs[
-                            "ex_data"
-                        ] = "node execution exceed rerun limit {}".format(rerun_limit)
+                        exec_outputs = self.runtime.get_execution_data_outputs(current_node_id)
+                        exec_outputs["ex_data"] = "node execution exceed rerun limit {}".format(rerun_limit)
 
-                        self.runtime.set_execution_data_outputs(
-                            current_node_id, exec_outputs
-                        )
+                        self.runtime.set_execution_data_outputs(current_node_id, exec_outputs)
                         self.runtime.set_state(
                             node_id=current_node_id,
                             to_state=states.FAILED,
@@ -547,10 +507,7 @@ class Engine:
                         return
 
                     # 设置状态前检测
-                    if (
-                        node_state.name
-                        not in states.INVERTED_TRANSITION[states.RUNNING]
-                    ):
+                    if node_state.name not in states.INVERTED_TRANSITION[states.RUNNING]:
                         self.runtime.sleep(process_id)
                         return
 
@@ -586,12 +543,11 @@ class Engine:
                     node_state,
                 )
                 handler = HandlerFactory.get_handler(node, self.runtime)
+                set_node_info(CurrentNodeInfo(node_id=current_node_id, version=version, loop=loop))
                 type_label = self._get_metrics_node_type(node)
                 execute_start = time.time()
                 execute_result = handler.execute(process_info, loop, version)
-                ENGINE_NODE_EXECUTE_TIME.labels(type_label).observe(
-                    time.time() - execute_start
-                )
+                ENGINE_NODE_EXECUTE_TIME.labels(type_label).observe(time.time() - execute_start)
 
                 # 进程是否要进入睡眠
                 if execute_result.should_sleep:
@@ -746,7 +702,7 @@ class Engine:
                     return
 
                 # engine context prepare
-                local.set_node_id(node_id)
+                set_node_info(CurrentNodeInfo(node_id=node_id, version=state.version, loop=state.loop))
 
                 # fetch callback data
                 callback_data = None
@@ -765,12 +721,8 @@ class Engine:
                     callback_data,
                 )
                 schedule_start = time.time()
-                schedule_result = handler.schedule(
-                    process_info, state.loop, schedule, callback_data
-                )
-                ENGINE_NODE_SCHEDULE_TIME.labels(type_label).observe(
-                    time.time() - schedule_start
-                )
+                schedule_result = handler.schedule(process_info, state.loop, schedule, callback_data)
+                ENGINE_NODE_SCHEDULE_TIME.labels(type_label).observe(time.time() - schedule_start)
 
                 if schedule_result.has_next_schedule:
                     self.runtime.set_next_schedule(
@@ -801,9 +753,7 @@ class Engine:
                 )
                 return
 
-            self.runtime.set_state(
-                node_id=node_id, to_state=states.FAILED, set_archive_time=True
-            )
+            self.runtime.set_state(node_id=node_id, to_state=states.FAILED, set_archive_time=True)
 
             outputs = self.runtime.get_execution_data_outputs(node_id)
             outputs["ex_data"] = ex_data
@@ -842,18 +792,12 @@ class Engine:
 
     def _ensure_state_is_fail_and_retrun_process_id(self, state: State) -> str:
         if state.name != states.FAILED:
-            raise InvalidOperationError(
-                "{} state is not FAILED, actual {}".format(state.node_id, state.name)
-            )
+            raise InvalidOperationError("{} state is not FAILED, actual {}".format(state.node_id, state.name))
 
         process_id = self.runtime.get_sleep_process_with_current_node_id(state.node_id)
 
         if not process_id:
-            raise InvalidOperationError(
-                "can not find sleep process with current node id: {}".format(
-                    state.node_id
-                )
-            )
+            raise InvalidOperationError("can not find sleep process with current node id: {}".format(state.node_id))
 
         return process_id
 
