@@ -239,9 +239,7 @@ class EngineAPIHooksMixin:
     """
 
     @abstractmethod
-    def pre_prepare_run_pipeline(
-        self, pipeline: dict, root_pipeline_data: dict, **options
-    ):
+    def pre_prepare_run_pipeline(self, pipeline: dict, root_pipeline_data: dict, **options):
         """
         调用 pre_prepare_run_pipeline 前执行的钩子
 
@@ -252,9 +250,7 @@ class EngineAPIHooksMixin:
         """
 
     @abstractmethod
-    def post_prepare_run_pipeline(
-        self, pipeline: dict, root_pipeline_data: dict, **options
-    ):
+    def post_prepare_run_pipeline(self, pipeline: dict, root_pipeline_data: dict, **options):
         """
         调用 pre_prepare_run_pipeline 后执行的钩子
 
@@ -428,7 +424,7 @@ class EngineAPIHooksMixin:
         """
 
     @abstractmethod
-    def post_forced_fail_activity(self, node_id: str, ex_data: str):
+    def post_forced_fail_activity(self, node_id: str, ex_data: str, old_version: str, new_version: str):
         """
         强制失败节点后执行的钩子
 
@@ -436,6 +432,10 @@ class EngineAPIHooksMixin:
         :type node_id: str
         :param ex_data: 写入节点执行数据的失败信息
         :type ex_data: str
+        :param old_version: 强制失败前的节点版本
+        :type old_version: str
+        :param new_version: 强制失败后的节点版本
+        :type new_version: str
         """
 
     @abstractmethod
@@ -527,9 +527,7 @@ class TaskMixin:
         """
 
     @abstractmethod
-    def start_timeout_monitor(
-        self, process_id: int, node_id: str, version: str, timeout: int
-    ):
+    def start_timeout_monitor(self, process_id: int, node_id: str, version: str, timeout: int):
         """
         开始对某个节点执行的超时监控，若超时时间归零后节点未进入归档状态，则强制失败该节点
 
@@ -644,9 +642,7 @@ class ProcessMixin:
         """
 
     @abstractmethod
-    def get_process_info_with_root_pipeline(
-        self, pipeline_id: str
-    ) -> List[ProcessInfo]:
+    def get_process_info_with_root_pipeline(self, pipeline_id: str) -> List[ProcessInfo]:
         """
         根据根流程 ID 获取一批进程的信息
 
@@ -666,9 +662,7 @@ class ProcessMixin:
         """
 
     @abstractmethod
-    def get_suspended_process_info(
-        self, suspended_by: str
-    ) -> List[SuspendedProcessInfo]:
+    def get_suspended_process_info(self, suspended_by: str) -> List[SuspendedProcessInfo]:
         """
         获取由于 pipeline 暂停而被暂停执行的进程信息
 
@@ -994,9 +988,7 @@ class ScheduleMixin:
         """
 
     @abstractmethod
-    def get_schedule_with_node_and_version(
-        self, node_id: str, version: str
-    ) -> Schedule:
+    def get_schedule_with_node_and_version(self, node_id: str, version: str) -> Schedule:
         """
         通过节点 ID 和执行版本来获取 Scheudle 对象
 
@@ -1088,9 +1080,7 @@ class ContextMixin:
         """
 
     @abstractmethod
-    def upsert_plain_context_values(
-        self, pipeline_id: str, update: Dict[str, ContextValue]
-    ):
+    def upsert_plain_context_values(self, pipeline_id: str, update: Dict[str, ContextValue]):
         """
         更新或创建新的普通上下文数据
 
@@ -1320,9 +1310,7 @@ class ExecutionHistoryMixin:
         """
 
     @abstractmethod
-    def get_short_histories(
-        self, node_id: str, loop: int = -1
-    ) -> List[ExecutionShortHistory]:
+    def get_short_histories(self, node_id: str, loop: int = -1) -> List[ExecutionShortHistory]:
         """
         返回某个节点的简要历史记录
 
@@ -1349,9 +1337,7 @@ class EngineRuntimeInterface(
     metaclass=ABCMeta,
 ):
     @abstractmethod
-    def prepare_run_pipeline(
-        self, pipeline: dict, root_pipeline_data: dict, **options
-    ) -> int:
+    def prepare_run_pipeline(self, pipeline: dict, root_pipeline_data: dict, **options) -> int:
         """
         进行 pipeline 执行前的准备工作，并返回 进程 ID，该函数执行完成后即代表
         pipeline 是随时可以通过 execute(process_id, start_event_id) 启动执行的
